#  tests for tensorflow-base-2.9.1-mkl_py37h6a7f48e_1 (this is a generated file);
print('===== testing package: tensorflow-base-2.9.1-mkl_py37h6a7f48e_1 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import tensorflow as tf
hello = tf.constant('Hello, TensorFlow!')
a = tf.constant(10)
b = tf.constant(32)
a + b
#  --- run_test.py (end) ---

print('===== tensorflow-base-2.9.1-mkl_py37h6a7f48e_1 OK =====');
print("import: 'tensorflow'")
import tensorflow

